
package jworms.game.explosion;

import jworms.game.GameField;
import jworms.game.WormUI;
import jworms.openGL.SmallTexture;
import jworms.util.MathUtil;
import org.lwjgl.opengl.GL;

import java.awt.*;
import java.awt.geom.Point2D;

public class FlameExplosionFlame extends ExplosionObject{
// static
  /** Flame textures. */
  private static SmallTexture textures_flame[] = null;

  /**
   * Loads textures and sound.
   */
  public static void init(){
    // load images
    try{
      textures_flame = WormUI.loadTextures("images/fire/flame");
    } catch(Exception e){
      System.out.println("Error in FlameExplosionFlame during loading images.");
      e.printStackTrace();
    }
  }

// non static
  /** Scale factor. */
  protected float scaleFactor = 1.5f;
  /** Alpha factor. */
  protected float alphaFactor = 1.0f;

  /** Current texture number. */
  protected float texture_nr = 0;

  /** Angle. */
  protected float angle = 0;
  /** Rotation. */
  protected float rotation = 0;

  /**
   * Creates an explosion flame.
   *
   * @param gameField game field
   * @param position position
   * @param motion motion
   */
  public FlameExplosionFlame(GameField gameField, Point2D.Float position, Point2D.Float motion){
    super(gameField, position, motion);

    maxLife = 10 + (float) (Math.random() - 0.5) * 10;
    rotation = (float) (Math.random() -0.5) / 20;
    texture_nr = (float) Math.random() * 16;
  }

  public boolean isFinished(){
    return lifeSeconds > maxLife;
  }

  protected void move(){
    if(gameField.getDot((int) position.x, (int) position.y) == GameField.DOTMAP_BACKGROUND){
      motion.y += GameField.GRAVITY;

      motion.x *= 0.98;
      motion.y *= 0.98;

      position.x += motion.x;
      position.y += motion.y;
    }else{
      motion.x = 0;
      motion.y = 0;
    }

    angle += rotation;
  }

  public void draw(){
    float alpha = MathUtil.inBounds(0, 1 - (lifeSeconds - maxLife + 3)/3, 1);

    GL.glColor4f(1.0f, 1.0f, 1.0f, alpha * alphaFactor);

    SmallTexture texture = getTexture();

    Point screen = gameField.fieldToScreen((int) position.x, (int) position.y);
    screen.x -= texture.getImage().getWidth() / 2;
    screen.y -= texture.getImage().getHeight() / 2;

    float scale = alpha * scaleFactor;
    texture.draw(screen.x, screen.y, scale, scale, (float) Math.toDegrees(angle));
  }

  /**
   * Returns the texture.
   *
   * @return texture
   */
  public SmallTexture getTexture(){
    texture_nr += Math.random() / 5;
    return textures_flame[(int) texture_nr % textures_flame.length];
  }

  public void dispose(){
  }
}
