
package jworms.game.weapons;

import jworms.game.GameField;
import jworms.game.GameObject;
import jworms.game.Player;

import java.awt.*;

/**
 * Every weapon should have a constructor with only the gameField as argument.
 *   public Weapon(GameField gameField);
 */
public interface Weapon{
  /**
   * Constructor for the weapon.
   *
   * @param player player that fires the weapon
   * @param gameField game field
   * @return weapon
   */
  public GameObject fireWeapon(Player player, GameField gameField);

  /**
   * Draws the weapon symbol for the given player.
   *
   * @param player player
   */
  public void drawSymbol(Player player);

  /**
   * Returns the current center of the weapon in GameField coordinates.
   *
   * @param player player
   * @return center of the weapon
   */
  public Point getCenter(Player player);

  /**
   * Returns true if the weapon needs aiming at the target.
   * (aiming means choosing the fireing angle)
   *
   * @return true if aiming weapon
   */
  public boolean needAiming();

  /**
   * Returns true if the weapon needs charging before fireing.
   * (charging means choosing the shoot strength)
   *
   * @return true if charging is needed
   */
  public boolean needsCharging();

  /**
   * Called once per game loop (frame).
   */
  public void loop();

  /**
   * Returns true if the weapon is ready for fireing.
   *
   * @return true if ready for fireing
   */
  public boolean isReady();

  /**
   * Returns reload percentage [0, 1].
   *
   * @return reload value (1 means reload is ready)
   */
  public float getReloadProcess();
}
