
package jworms.openGL;

import org.lwjgl.Display;
import org.lwjgl.DisplayMode;
import org.lwjgl.Sys;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.GL;
import org.lwjgl.opengl.GLU;
import org.lwjgl.opengl.Window;

/**
 * Utils for initializing and working with OpenGL.
 */
public class OpenGLUtils{
  /**
   * Initializes fullscreen mode with the given width and heigt.
   *
   * @param title window title
   */
  public static void initFullscreen(String title){
    // set display mode and create fullscreen window
    try{
      Window.create(title, 24, 8, 0, 0);
      System.out.println("Created OpenGL.");
    } catch(Exception e){
      System.err.println("Failed to create OpenGL due to " + e);
      System.exit(1);
    }

    try{
      Keyboard.create();
      Keyboard.enableBuffer();
      Mouse.create();
      Sys.setTime(0);
      Sys.setProcessPriority(Sys.HIGH_PRIORITY);
    } catch(Exception e){
      System.err.println("Failed to create Keybourd and Mouse due to " + e);
      System.exit(1);
    }

    // enable texture mapping
    GL.glEnable(GL.GL_TEXTURE_2D);

    // alpha test/ blending
    GL.glAlphaFunc(GL.GL_GREATER, 0.5f);
    GL.glEnable(GL.GL_ALPHA_TEST);
    GL.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
    GL.glEnable(GL.GL_BLEND);

    GL.glDisable(GL.GL_DEPTH_TEST);

    // set up 2D mode
    GL.glMatrixMode(GL.GL_PROJECTION);
    GL.glLoadIdentity();
    GLU.gluOrtho2D(0, Display.getWidth(), Display.getHeight(), 0);
    GL.glMatrixMode(GL.GL_MODELVIEW);
    GL.glLoadIdentity();
    GL.glViewport(0, 0, Display.getWidth(), Display.getHeight());
  }

  /**
   * Tries to set the display mode.
   *
   * @param width width
   * @param height height
   */
  public static void setDisplayMode(int width, int height){
    // choose display mode
    DisplayMode displayMode = null;

    DisplayMode[] modes = Display.getAvailableDisplayModes();
    for(int mode_nr = 0; mode_nr < modes.length; mode_nr++){
      DisplayMode mode = modes[mode_nr];

      if(mode.width == width && mode.height == height && mode.bpp >= 24){
        displayMode = mode;
        break;
      }
    }

    // set display mode and create fullscreen window
    try{
      if(displayMode != null){
        System.out.println("Setting up display mode " + displayMode);
        Display.setDisplayMode(displayMode);
      }
    } catch(Exception e){
      System.err.println("Failed to set dispay mode due to " + e);
      System.exit(1);
    }
  }

  /**
   * Destroys the OpenGL window and frees all resources.
   */
  public static void destroy(){
    Keyboard.destroy();
    Mouse.destroy();
    Window.destroy();
    try{
      Display.resetDisplayMode();
    } catch(Exception e){
      e.printStackTrace();
    }
  }

  /**
   * Returns true if the window should be closed.
   *
   * @return true if the window should be closed
   */
  public static boolean isCloseRequested(){
    return Window.isCloseRequested();
  }
}
